@@ -2,22 +2,29 @@
 #[allow(unused_imports)]
 use crate as ts3;
 
-use crate::event::{EventHandler, Handler};
-use crate::{Decode, Error, BoxError};
+pub use async_trait::async_trait;
+
+use crate::{
+    event::{EventHandler, Handler},
+    BoxError, CommandBuilder, Decode, Error, Serialize,
+};
 use bytes::Bytes;
-use std::collections::HashMap;
-use std::convert::From;
-use std::fmt::{self, Display, Formatter};
-use std::result;
-use std::str::from_utf8;
-use std::sync::{Arc, RwLock};
-use std::time::Duration;
-use tokio::io::{AsyncBufReadExt, AsyncWriteExt, BufReader};
-use tokio::net::TcpStream;
-use tokio::net::ToSocketAddrs;
-use tokio::sync::{mpsc, oneshot};
-use tokio::task::spawn;
-use tokio::time::sleep;
+use std::{
+    collections::HashMap,
+    convert::From,
+    fmt::{self, Display, Formatter, Write},
+    result,
+    str::from_utf8,
+    sync::{Arc, RwLock},
+    time::Duration,
+};
+use tokio::{
+    io::{AsyncBufReadExt, AsyncWriteExt, BufReader},
+    net::{TcpStream, ToSocketAddrs},
+    sync::{mpsc, oneshot},
+    task::spawn,
+    time::sleep,
+};
 
 pub type Result<T> = result::Result<T, Error>;
 
@@ -240,11 +247,15 @@ impl Default for APIKeyScope {
 impl Display for APIKeyScope {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         use APIKeyScope::*;
-        write!(f, "{}", match self {
-            Manage => "manage",
-            Write => "writer",
-            Read => "read",
-        })
+        write!(
+            f,
+            "{}",
+            match self {
+                Manage => "manage",
+                Write => "writer",
+                Read => "read",
+            }
+        )
     }
 }
 
@@ -254,7 +265,7 @@ impl Decode<APIKeyScope> for APIKeyScope {
             "manage" => Self::Manage,
             "write" => Self::Write,
             "read" => Self::Read,
-            s => panic!("Unexpected enum variant for APIKeyScope: {}", s)
+            s => panic!("Unexpected enum variant for APIKeyScope: {}", s),
         })
     }
 }
@@ -290,11 +301,11 @@ pub enum TextMessageTarget {
     Server,
 }
 
-impl Display for TextMessageTarget {
-    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+impl Serialize for TextMessageTarget {
+    fn serialize(&self, writer: &mut String) {
         use TextMessageTarget::*;
         write!(
-            f,
+            writer,
             "{}",
             match self {
                 Client(clid) => format!("1 target={}", clid),
@@ -302,6 +313,7 @@ impl Display for TextMessageTarget {
                 Server => "3".to_owned(),
             }
         )
+        .unwrap();
     }
 }
 
@@ -310,14 +322,25 @@ impl Client {
     /// Creates a new apikey using the specified scope, for the invoking user. The default
     /// lifetime of a token is 14 days, a zero lifetime means no expiration. It is possible
     ///  to create apikeys for other users using `b_virtualserver_apikey_manage.`
-    pub async fn apikeyadd(&self, scope: APIKeyScope, lifetime: Option<u64>, cldbid: Option<u64>) -> Result<APIKey> {
-        self.send(format!("apikeyadd scope={} {} {}", scope, match lifetime {
-            None => "".to_owned(),
-            Some(lifetime) => format!("lifetime={}", lifetime)
-        }, match cldbid {
-            None => "".to_owned(),
-            Some(cldbid) => format!("cldbid={}", cldbid)
-        })).await
+    pub async fn apikeyadd(
+        &self,
+        scope: APIKeyScope,
+        lifetime: Option<u64>,
+        cldbid: Option<u64>,
+    ) -> Result<APIKey> {
+        self.send(format!(
+            "apikeyadd scope={} {} {}",
+            scope,
+            match lifetime {
+                None => "".to_owned(),
+                Some(lifetime) => format!("lifetime={}", lifetime),
+            },
+            match cldbid {
+                None => "".to_owned(),
+                Some(cldbid) => format!("cldbid={}", cldbid),
+            }
+        ))
+        .await
     }
 
     /// Delete an apikey. Any apikey owned by the current user can always be deleted. Deleting
@@ -328,23 +351,65 @@ impl Client {
 
     /// Lists all apikeys owned by the user, or of all users using `cldbid`=`(0, true).` Usage
     /// of `cldbid`=... requires `b_virtualserver_apikey_manage`.
-    pub async fn apikeylist(&self, cldbid: Option<(u64, bool)>, start: Option<u64>, duration: Option<u64>, count: bool) -> Result<Vec<APIKey>> {
-        self.send(format!("apikeylist {} {} {} {}", match cldbid {
-            None => "".to_owned(),
-            Some((cldbid, all)) => format!("cldbid={}", match all {
-                true => "*".to_owned(),
-                false => cldbid.to_string(),
-            })
-        }, match start {
-            None => "".to_owned(),
-            Some(start) => format!("start={}", start),
-        }, match duration {
-            None => "".to_owned(),
-            Some(duration) => format!("duration={}", duration),
-        }, match count {
-            true => "-count",
-            false => "",
-        })).await
+    pub async fn apikeylist(
+        &self,
+        cldbid: Option<(u64, bool)>,
+        start: Option<u64>,
+        duration: Option<u64>,
+        count: bool,
+    ) -> Result<Vec<APIKey>> {
+        self.send(format!(
+            "apikeylist {} {} {} {}",
+            match cldbid {
+                None => "".to_owned(),
+                Some((cldbid, all)) => format!(
+                    "cldbid={}",
+                    match all {
+                        true => "*".to_owned(),
+                        false => cldbid.to_string(),
+                    }
+                ),
+            },
+            match start {
+                None => "".to_owned(),
+                Some(start) => format!("start={}", start),
+            },
+            match duration {
+                None => "".to_owned(),
+                Some(duration) => format!("duration={}", duration),
+            },
+            match count {
+                true => "-count",
+                false => "",
+            }
+        ))
+        .await
+    }
+
+    /// Add a new ban rule on the selected virtual server. One of `ip`, `name`, `uid`
+    /// and `mytsid` must not be `None`.
+    pub async fn banadd(
+        &self,
+        ip: Option<&str>,
+        name: Option<&str>,
+        uid: Option<&str>,
+        mytsid: Option<&str>,
+        time: Option<u64>,
+        banreason: Option<&str>,
+        lastnickname: Option<&str>,
+    ) -> Result<()> {
+        self.send({
+            CommandBuilder::new("banadd")
+                .arg_opt("ip", ip)
+                .arg_opt("name", name)
+                .arg_opt("uid", uid)
+                .arg_opt("mytsid", mytsid)
+                .arg_opt("time", time)
+                .arg_opt("banreason", banreason)
+                .arg_opt("lastnickname", lastnickname)
+                .into_inner()
+        })
+        .await
     }
 
     /// Sends a text message to all clients on all virtual servers in the TeamSpeak 3
@@ -355,10 +420,12 @@ impl Client {
 
     /// Authenticate with the given data.
     pub async fn login(&self, username: &str, password: &str) -> Result<()> {
-        self.send(format!(
-            "login client_login_name={} client_login_password={}",
-            username, password
-        ))
+        self.send(
+            CommandBuilder::new("login")
+                .arg("client_login_name", username)
+                .arg("client_login_password", password)
+                .into_inner(),
+        )
         .await?;
         Ok(())
     }
@@ -376,18 +443,25 @@ impl Client {
     }
 
     pub async fn sendtextmessage(&self, target: TextMessageTarget, msg: &str) -> Result<()> {
-        self.send(format!("sendtextmessage targetmode={} msg={}", target, msg))
-            .await?;
+        self.send(
+            CommandBuilder::new("sendtextmessage")
+                .arg("targetmode", target)
+                .arg("msg", msg)
+                .into_inner(),
+        )
+        .await?;
         Ok(())
     }
 
     /// Adds one or more clients to the server group specified with sgid. Please note that a
     /// client cannot be added to default groups or template groups.
     pub async fn servergroupaddclient(&self, sgid: usize, cldbid: usize) -> Result<()> {
-        self.send(format!(
-            "servergroupaddclient sgid={} cldbid={}",
-            sgid, cldbid
-        ))
+        self.send(
+            CommandBuilder::new("servergroupaddclient")
+                .arg("sgid", sgid)
+                .arg("cldbid", cldbid)
+                .into_inner(),
+        )
         .await?;
         Ok(())
     }
@@ -395,10 +469,12 @@ impl Client {
     /// Removes one or more clients specified with cldbid from the server group specified with
     /// sgid.  
     pub async fn servergroupdelclient(&self, sgid: usize, cldbid: usize) -> Result<()> {
-        self.send(format!(
-            "servergroupdelclient sgid={} cldbid={}",
-            sgid, cldbid
-        ))
+        self.send(
+            CommandBuilder::new("servergroupdelclient")
+                .arg("sgid", sgid)
+                .arg("cldbid", cldbid)
+                .into_inner(),
+        )
         .await?;
         Ok(())
     }
@@ -454,7 +530,7 @@ impl Client {
     }
 }
 
-/// An API Key returned from [`apikeyadd`].
+/// An API Key returned from [`Client.apikeyadd`].
 #[derive(Debug, Decode, Default)]
 pub struct APIKey {
     pub apikey: String,
