@@ -1,8 +1,11 @@
-//! # ts3
-//! A WIP ts3 query interface library
+//! # TS3
+//! A fully asynchronous library to interact with the TeamSpeak 3 Server query interface.
+//! The commands are avaliable after connecting to a TS3 Server using a [`Client`]. Commands
+//! can either be sent using the associated command or using [`Client.sent`] to send raw messages.
 //!
 //! # Examples
 //!
+//! Connect to a TS3 query interface and select a server
 //! ```rust
 //! use ts3::Client;
 //!
@@ -17,21 +20,55 @@
 //!     Ok(())
 //! }
 //! ```
+//!
+//! ```rust
+//! use ts3::{
+//!     async_trait, Client,
+//!     client::TextMessageTarget,
+//!     event::{EventHandler, ClientEnterView}
+//! };
+//!
+//! #[tokio::main]
+//! async fn main() -> Result<(), Box<dyn std::error::Error + Send + Sync>> {
+//!     let client = Client::new("localhost:10011").await?;
+//!
+//!     client.use_sid(1).await?;
+//!
+//!     // Assign a new event handler.
+//!     client.set_event_handler(Handler);
+//!
+//!     tokio::signal::ctrl_c().await?;
+//! }
+//!
+//! pub struct Handler;
+//!
+//! #[async_trait]
+//! impl EventHandler for Handler {
+//!     async fn cliententerview(&self, client: Client, event: ClientEnterView) {
+//!         println!("Client {} joined!", event.client_nickname);
+//!
+//!         // Send a private message to the client using "sendtextmessage".
+//!         client.sendtextmessage(TextMessageTarget::Client(event.clid), "Hello World!")
+//!             .await.unwrap();
+//!     }
+//! }
+//!
+//! ```
 
 pub mod client;
 pub mod event;
-mod macros;
 
 pub use client::{Client, RawResp};
 pub use event::EventHandler;
 pub use ts3_derive::Decode;
 
-use std::convert::TryFrom;
-use std::error;
-use std::fmt::Debug;
-use std::fmt::{self, Display, Formatter};
-use std::io;
-use std::str::{from_utf8, FromStr};
+use std::{
+    convert::TryFrom,
+    error,
+    fmt::{self, Debug, Display, Formatter, Write},
+    io,
+    str::{from_utf8, FromStr},
+};
 
 pub enum ParseError {
     InvalidEnum,
@@ -169,6 +206,10 @@ pub trait Decode<T> {
     fn decode(buf: &[u8]) -> Result<T, BoxError>;
 }
 
+pub trait Serialize {
+    fn serialize(&self, writer: &mut String);
+}
+
 /// Implement `Decode` for `Vec` as long as `T` itself also implements `Decode`.
 impl<T> Decode<Vec<T>> for Vec<T>
 where
@@ -185,6 +226,17 @@ where
     }
 }
 
+/// Implements `Serialize` for types that can be directly written as they are formatted.
+macro_rules! impl_serialize {
+    ($t:ty) => {
+        impl crate::Serialize for $t {
+            fn serialize(&self, writer: &mut ::std::string::String) {
+                write!(writer, "{}", self).unwrap();
+            }
+        }
+    };
+}
+
 /// The `impl_decode` macro implements `Decode` for any type that implements `FromStr`.
 macro_rules! impl_decode {
     ($t:ty) => {
@@ -196,10 +248,50 @@ macro_rules! impl_decode {
     };
 }
 
+#[derive(Clone, Debug, Default)]
+pub struct CommandBuilder(String);
+
+impl CommandBuilder {
+    pub fn new<T>(name: T) -> Self
+    where
+        T: ToString,
+    {
+        Self(name.to_string())
+    }
+
+    pub fn arg<T, S>(mut self, key: T, value: S) -> Self
+    where
+        T: AsRef<str>,
+        S: Serialize,
+    {
+        self.0.write_char(' ').unwrap();
+        self.0.write_str(key.as_ref()).unwrap();
+        self.0.write_char('=').unwrap();
+        value.serialize(&mut self.0);
+        self
+    }
+
+    pub fn arg_opt<T, S>(self, key: T, value: Option<S>) -> Self
+    where
+        T: AsRef<str>,
+        S: Serialize,
+    {
+        match value {
+            Some(value) => self.arg(key, value),
+            None => self,
+        }
+    }
+
+    pub fn into_inner(self) -> String {
+        self.0
+    }
+}
+
 /// Implement `Decode` for `()`. Calling `()::decode(&[u8])` will never fail
 /// and can always be unwrapped safely.
 impl Decode<()> for () {
     fn decode(_: &[u8]) -> Result<(), BoxError> {
+        // command!("");
         Ok(())
     }
 }
@@ -247,6 +339,41 @@ impl Decode<String> for String {
     }
 }
 
+impl Serialize for &str {
+    fn serialize(&self, writer: &mut String) {
+        for c in self.chars() {
+            match c {
+                '\\' => writer.write_str("\\\\").unwrap(),
+                '/' => writer.write_str("\\/").unwrap(),
+                ' ' => writer.write_str("\\s").unwrap(),
+                '|' => writer.write_str("\\p").unwrap(),
+                c if c == 7u8 as char => writer.write_str("\\a").unwrap(),
+                c if c == 8u8 as char => writer.write_str("\\b").unwrap(),
+                c if c == 12u8 as char => writer.write_str("\\f").unwrap(),
+                c if c == 10u8 as char => writer.write_str("\\n").unwrap(),
+                c if c == 13u8 as char => writer.write_str("\\r").unwrap(),
+                c if c == 9u8 as char => writer.write_str("\\t").unwrap(),
+                c if c == 11u8 as char => writer.write_str("\\v").unwrap(),
+                _ => writer.write_char(c).unwrap(),
+            }
+        }
+    }
+}
+
+impl Serialize for bool {
+    fn serialize(&self, writer: &mut String) {
+        write!(
+            writer,
+            "{}",
+            match self {
+                false => b'0',
+                true => b'1',
+            }
+        )
+        .unwrap();
+    }
+}
+
 impl Decode<bool> for bool {
     fn decode(buf: &[u8]) -> Result<bool, BoxError> {
         match buf.get(0) {
@@ -275,6 +402,20 @@ impl_decode!(u32);
 impl_decode!(u64);
 impl_decode!(u128);
 
+impl_serialize!(isize);
+impl_serialize!(i8);
+impl_serialize!(i16);
+impl_serialize!(i32);
+impl_serialize!(i64);
+impl_serialize!(i128);
+
+impl_serialize!(usize);
+impl_serialize!(u8);
+impl_serialize!(u16);
+impl_serialize!(u32);
+impl_serialize!(u64);
+impl_serialize!(u128);
+
 impl Decode<Error> for Error {
     fn decode(buf: &[u8]) -> Result<Error, BoxError> {
         let (mut id, mut msg) = (0, String::new());
@@ -365,4 +506,25 @@ mod tests {
             }
         );
     }
+
+    #[test]
+    fn test_command_builder() {
+        let cmd = CommandBuilder::new("testcmd");
+        assert_eq!(cmd.clone().into_inner(), "testcmd");
+
+        let cmd = cmd.arg("hello", "world");
+        assert_eq!(cmd.clone().into_inner(), "testcmd hello=world");
+
+        let cmd = cmd.arg("test", "1234");
+        assert_eq!(cmd.clone().into_inner(), "testcmd hello=world test=1234");
+
+        let cmd = cmd.arg_opt("opt", Some("arg"));
+        assert_eq!(
+            cmd.clone().into_inner(),
+            "testcmd hello=world test=1234 opt=arg"
+        );
+
+        let cmd = cmd.arg_opt::<_, &str>("opt2", None);
+        assert_eq!(cmd.into_inner(), "testcmd hello=world test=1234 opt=arg");
+    }
 }
