@@ -1,11 +1,10 @@
-use ts3::{Client, Decode};
 use async_trait::async_trait;
+use ts3::{Client, Decode};
 
 #[tokio::main]
 async fn main() {
     let password = "bN5cSqv+";
 
-
     let client = Client::new("localhost:10011").await.unwrap();
 
     client.set_event_handler(Handler);
@@ -13,7 +12,10 @@ async fn main() {
     client.login("serveradmin", password).await.unwrap();
     client.use_sid(1).await.unwrap();
 
-    client.servernotifyregister(ts3::client::ServerNotifyRegister::TextPrivate).await.unwrap();
+    client
+        .servernotifyregister(ts3::client::ServerNotifyRegister::TextPrivate)
+        .await
+        .unwrap();
 
     loop {}
 }
